kafka-humio-gateway
===================

This project acts as a Kafka consumer and feeds events to a Humio ingestion endpoint.

Configuration
-------------

The configuration file is a simple Yaml file:

```
kafka:
  brokers:
    - broker1:9092
    - broker2:9092
    - ...
  topics:
    - event-named-in-plugin-parameters-1
    - event-named-in-plugin-parameters-2
    - ...
  consumer_group: unique-consumer-group-name
humio:
  endpoint_url: https://cloud.community.humio.com:443/api/v1/ingest/humio-structured
  ingest_token: your-ingest-token
  batching_timeout_ms: timeout-between-outbound-requests-in-milliseconds
  event_batch_size: count-of-events-before-pushing

```

Every field except `consumer_group`, `batching_timeout_ms`, and `event_batch_size` is required.
These fields use the following defaults:
- `consumer_group: velociraptor-consumer`
- `batching_timeout_ms: 3000`
- `event_batch_size: 500`

Data Format
-----------

The JSON provided by the Kafka exporter in Velociraptor has the following schema:
{
	"version" : 1,
	"row_data" :
		{ ... attributes },
	"tags" :
		{ ... zero or more tags }
}

The row data is provided as generated by the query.  The version is hard-coded by Velociraptor.  As of this writing there is only version 1.  If the query was generated by a client artifact, tags will automatically contain "ClientId" and "ClientHostname" tags.  Other tags can be provided using the parameters to the `kafka_publisher` plugin in Velociraptor.

The Humio structured data API requires events be POSTed with the following JSON schema.  The structure containing both tags and events is referred to here as a payload.  The Humio API accepts one or more payloads in a JSON array. The API also accepts multiple events in the same payload but, as of this writing, we do not use that functionality as it requires that the tags be identical.  Since tags are arbitrary data, comparing them client-side just to batch events into a single payload doesn't bring much value.  The tag and attribute names must be strings.  The values can be either strings or numeric.


```
[{
	"tags" : {
		"tag1" : "value1",
		"tag2" : "value2"
	}, events : [
		{
			"attributes" : { ... attributes ... },
			"timestamp: : <timestamp>,
			"timezone" : "UTC" # Only if timestamp is seconds/milliseconds from epoch
		},
		{ zero or more events  }
	]
}, { zero or more payload }]
```
